@@ -1,4 +1,4 @@
-/* Copyright (c) 1998, 1999, 2000, 2001 Thorsten Kukuk
+/* Copyright (c) 1998, 1999, 2000, 2001, 2002 Thorsten Kukuk
    This file is part of ypbind-mt.
    Author: Thorsten Kukuk <kukuk@suse.de>
 
@@ -87,7 +87,7 @@ static pthread_rdwr_t domainlock = PTHREAD_RDWR_INITIALIZER;
 static pthread_mutex_t search_lock = PTHREAD_MUTEX_INITIALIZER;
 
 static void do_broadcast (struct binding *list);
-static void ping_all (struct binding *list);
+static int ping_all (struct binding *list);
 
 /* this is called from the RPC thread (ypset). */
 void
@@ -200,10 +200,9 @@ find_domain (const char *domain, ypbind_resp *result)
 		{
 		  if (debug_flag)
 		    log_msg (LOG_DEBUG, "trylock = success");
-		  if (domainlist[i].use_broadcast)
+		  if (!ping_all (&domainlist[i]) &&
+		      domainlist[i].use_broadcast)
 		    do_broadcast (&domainlist[i]);
-		  else
-		    ping_all (&domainlist[i]);
 		  pthread_mutex_unlock (&search_lock);
 		  ++second;
 		  /* Get the read lock again for next run. */
@@ -283,7 +282,7 @@ update_bindingfile (struct binding *entry)
   len = iov[0].iov_len + iov[1].iov_len;
 
   if ((fd = open(path1, O_CREAT | O_RDWR | O_TRUNC, FILE_MODE )) != -1)
-    {  
+    {
       if (writev (fd, iov, 2) != len )
         {
           log_msg (LOG_ERR, "writev (%s): %s", path1, strerror (errno));
@@ -370,11 +369,12 @@ get_entry (const char *domain, struct binding **entry)
   return 0;
 }
 
-void
-add_server (const char *domain, const char *host)
+int
+add_server (const char *domain, const char *host, int check_syntax)
 {
   struct binding *entry;
   int active;
+  int res = 0;
 
   if (domain == NULL)
     {
@@ -390,14 +390,24 @@ add_server (const char *domain, const char *host)
   if (host == NULL)
     {
       entry->use_broadcast = TRUE;
+      res = 1;
 
       if (debug_flag)
 	log_msg (LOG_DEBUG,
-		 _("add_server() domain: %s, broadcast, slot: 0"),
+		 _("add_server() domain: %s, broadcast"),
 		 domain);
     }
   else
     {
+      struct hostent *hent;
+#if defined (HAVE___NSS_CONFIGURE_LOOKUP)
+      struct hostent hostbuf;
+      size_t hstbuflen;
+      char *hsttmpbuf;
+      int herr;
+      int error;
+#endif
+
       /* find empty slot */
       for (active = 0; active < _MAXSERVER; ++active)
 	if (entry->server[active].host == NULL)
@@ -405,89 +415,95 @@ add_server (const char *domain, const char *host)
 
       /* There is no empty slot */
       if (entry->server[active].host != NULL)
-	goto exit;
+	{
+	  if (debug_flag)
+	    log_msg (LOG_DEBUG,
+		     _("add_server() domain: %s, host: %s, NO SLOT FREE!"),
+		     domain, host);
+	  goto exit;
+	}
 
       if (debug_flag)
 	log_msg (LOG_DEBUG,
-		 _("add_server() domain: %s, host: %s, %sbroadcast, slot: %d"),
-		 domain, host ? host : _("unknown"), (host == NULL) ? "" :
-		 _("no"), active);
+		 _("add_server() domain: %s, host: %s, slot: %d"),
+		 domain, host, active);
 
-      if (host != NULL)
-	{
-	  struct hostent *hent;
 #if defined (HAVE___NSS_CONFIGURE_LOOKUP)
-	  struct hostent hostbuf;
-	  size_t hstbuflen;
-	  char *hsttmpbuf;
-	  int herr;
-	  int error;
-#endif
-	  entry->server[active].host = strdup(host);
-#if defined (HAVE___NSS_CONFIGURE_LOOKUP)
-	  hstbuflen = 1024;
-	  hsttmpbuf = alloca (hstbuflen);
-	  while ((error= gethostbyname_r (entry->server[active].host,
-					  &hostbuf, hsttmpbuf, hstbuflen,
-					  &hent, &herr)) != 0)
-	    if (herr == NETDB_INTERNAL || (error == -1 && errno == ERANGE)
-		|| error == ERANGE)
-	      {
-		/* Enlarge the buffer.  */
-		hstbuflen *= 2;
-		hsttmpbuf = alloca (hstbuflen);
-	      }
-	    else
-	      break;
-#else
-#if defined(HAVE_RES_GETHOSTBYNAME)
-	  hent = res_gethostbyname (entry->server[active].host);
+      hstbuflen = 1024;
+      hsttmpbuf = alloca (hstbuflen);
+      while ((error= gethostbyname_r (host, &hostbuf, hsttmpbuf, hstbuflen,
+				      &hent, &herr)) != 0)
+	if (herr == NETDB_INTERNAL || (error == -1 && errno == ERANGE)
+	    || error == ERANGE)
+	  {
+	    /* Enlarge the buffer.  */
+	    hstbuflen *= 2;
+	    hsttmpbuf = alloca (hstbuflen);
+	  }
+	else
+	  break;
+#elif defined(HAVE_RES_GETHOSTBYNAME)
+      hent = res_gethostbyname (host);
 #elif defined(HAVE__DNS_GETHOSTBYNAME)
-	  hent = _dns_gethostbyname (entry->server[active].host);
+      hent = _dns_gethostbyname (host);
 #else
-	  hent = gethostbyname (entry->server[active].host);
+      hent = gethostbyname (host);
 #endif
-#endif
-	  if (!hent)
+      if (!hent)
+	{
+	  switch (h_errno)
 	    {
-	      switch (h_errno)
-		{
-		case HOST_NOT_FOUND:
-		  log_msg (LOG_ERR, _("Unknown host: %s"),
-			   entry->server[active].host);
-		  break;
-		case TRY_AGAIN:
-		  log_msg (LOG_ERR, _("Host name lookup failure"));
-		  break;
-		case NO_DATA:
-		  log_msg (LOG_ERR, _("No address associated with name: %s"),
-			   entry->server[active].host);
-		  break;
-		case NO_RECOVERY:
-		  log_msg (LOG_ERR, _("Unknown server error"));
-		  break;
-		default:
-		  log_msg (LOG_ERR, _("gethostbyname: Unknown error"));
-		  break;
-		}
-	      return;
+	    case HOST_NOT_FOUND:
+	      if (check_syntax)
+		fprintf (stderr, "%s %s\n", _("Unknown host:"), host);
+	      else
+		log_msg (LOG_ERR, "%s %s", _("Unknown host:"), host);
+	      break;
+	    case TRY_AGAIN:
+	      if (check_syntax)
+		fprintf (stderr, "%s\n", _("Host name lookup failure"));
+	      else
+		log_msg (LOG_ERR, _("Host name lookup failure"));
+	      break;
+	    case NO_DATA:
+	      if (check_syntax)
+		fprintf (stderr, "%s %s\n",
+			 _("No address associated with name:"), host);
+	      else
+		log_msg (LOG_ERR, "%s %s",
+			 _("No address associated with name:"), host);
+	      break;
+	    case NO_RECOVERY:
+	      if (check_syntax)
+		fprintf (stderr, "%s\n", _("Unknown server error"));
+	      else
+		log_msg (LOG_ERR, _("Unknown server error"));
+	      break;
+	    default:
+	      if (check_syntax)
+		fprintf (stderr, "%s\n", _("gethostbyname: Unknown error"));
+	      else
+		log_msg (LOG_ERR, _("gethostbyname: Unknown error"));
+	      break;
 	    }
-	  if (hent->h_addr_list[0] == NULL)
-	    return;
-	  entry->server[active].family = hent->h_addrtype;
-	  /* XXX host could have multiple interfaces. We should
-	     try to use the interface on the local network.
-	     If there is none, use the first one. */
-	  memcpy (&entry->server[active].addr, hent->h_addr_list[0],
-		  hent->h_length);
+	  goto exit;
 	}
-      else
-	entry->server[active].host = NULL;
+      if (hent->h_addr_list[0] == NULL)
+	goto exit;
+
+      entry->server[active].host = strdup (host);
+      entry->server[active].family = hent->h_addrtype;
+      /* XXX host could have multiple interfaces. We should
+	 try to use the interface on the local network.
+	 If there is none, use the first one. */
+      memcpy (&entry->server[active].addr, hent->h_addr_list[0],
+	      hent->h_length);
+      res = 1;
     }
 
  exit:
   pthread_rdwr_wunlock_np (&domainlock);
-  return;
+  return res;
 }
 
 static struct binding *in_use = NULL;
@@ -585,6 +601,10 @@ do_broadcast (struct binding *list)
   bool_t out;
   enum clnt_stat status;
 
+  if (debug_flag)
+    log_msg (LOG_DEBUG, _("do_broadcast() for domain '%s' is called"),
+	     domain);
+
   pthread_rdwr_wlock_np (&domainlock);
   list->active = -1;
   pthread_rdwr_wunlock_np (&domainlock);
@@ -606,6 +626,10 @@ do_broadcast (struct binding *list)
       update_bindingfile (list);
       pthread_rdwr_runlock_np (&domainlock);
     }
+
+  if (debug_flag)
+    log_msg (LOG_DEBUG, _("leave do_broadcast() for domain '%s'"),
+	     domain);
 }
 
 #if USE_BROADCAST
@@ -684,7 +708,9 @@ struct findserv_req
   struct sockaddr_in sin;
 };
 
-static void
+/* This function sends a ping to every known ypserver. It returns 0,
+   if no running server is found, 1 else. */
+static int
 ping_all (struct binding *list)
 {
   const struct timeval TIMEOUT50 = {5, 0};
@@ -700,13 +726,16 @@ ping_all (struct binding *list)
   struct cu_data *cu;
   char *domain = list->domain;
 
+  if (list->server[0].host == NULL) /* There is no known server */
+    return 0;
+
   pthread_rdwr_wlock_np (&domainlock);
   list->active = -1;
   pthread_rdwr_wunlock_np (&domainlock);
 
   pings = malloc (sizeof (struct findserv_req *) * _MAXSERVER);
   if (pings == NULL)
-    return;
+    return 0;
   xid_seed = (u_int32_t) (time (NULL) ^ getpid ());
 
   for (i = 0; list->server[i].host; ++i)
@@ -743,7 +772,7 @@ ping_all (struct binding *list)
   if (pings_count == 0)
     {
       free (pings);
-      return;
+      return 0;
     }
 
   /* Create RPC handle */
@@ -755,7 +784,7 @@ ping_all (struct binding *list)
       for (i = 0; i < pings_count; ++i)
         free (pings[i]);
       free (pings);
-      return;
+      return 0;
     }
   clnt->cl_auth = authunix_create_default ();
   cu = (struct cu_data *) clnt->cl_private;
@@ -816,12 +845,12 @@ ping_all (struct binding *list)
   if (!found)
     remove_bindingfile(list->domain);
 
-  return;
+  return found;
 }
 
 #else /* Don't send a ping to all server at the same time */
 
-static void
+static int
 ping_all (struct binding *list)
 {
   char *domain = list->domain;
@@ -833,6 +862,9 @@ ping_all (struct binding *list)
   CLIENT *clnt_handlep = NULL;
   int i = 0;
 
+  if (list->server[0].host == NULL) /* There is no known server */
+    return 0;
+
   pthread_rdwr_wlock_np (&domainlock);
   list->active = -1;
   pthread_rdwr_wunlock_np (&domainlock);
@@ -898,16 +930,17 @@ ping_all (struct binding *list)
           pthread_rdwr_rlock_np (&domainlock);
           update_bindingfile (list);
           pthread_rdwr_runlock_np (&domainlock);
-          return;
+          return 1;
         }
 
       ++i;
       if (i == _MAXSERVER)
         {
           remove_bindingfile(list->domain);
-          return;
+          return 0;
         }
     }
+  return 0;
 }
 
 #endif
@@ -920,10 +953,8 @@ do_binding (void)
   pthread_mutex_lock (&search_lock);
   for (i = 0; i < max_domains; ++i)
     {
-      if (domainlist[i].use_broadcast)
+      if (!ping_all (&domainlist[i]) && domainlist[i].use_broadcast)
 	do_broadcast (&domainlist[i]);
-      else
-	ping_all (&domainlist[i]);
     }
   pthread_mutex_unlock (&search_lock);
 }
@@ -1042,10 +1073,9 @@ test_bindings (void *param __attribute__ ((unused)))
 		     the read lock again */
 		  pthread_rdwr_wunlock_np (&domainlock);
 		  pthread_mutex_lock (&search_lock);
-		  if (domainlist[i].use_broadcast)
+		  if (!ping_all (&domainlist[i]) &&
+		      domainlist[i].use_broadcast)
 		    do_broadcast (&domainlist[i]);
-		  else
-		    ping_all (&domainlist[i]);
 		  pthread_mutex_unlock (&search_lock);
 		  pthread_rdwr_rlock_np (&domainlock);
 		}
@@ -1056,10 +1086,8 @@ test_bindings (void *param __attribute__ ((unused)))
 		 server */
 	      pthread_rdwr_runlock_np (&domainlock);
 	      pthread_mutex_lock (&search_lock);
-	      if (domainlist[i].use_broadcast)
+	      if (!ping_all (&domainlist[i]) && domainlist[i].use_broadcast)
 		do_broadcast (&domainlist[i]);
-	      else
-		ping_all (&domainlist[i]);
 	      pthread_mutex_unlock (&search_lock);
 	      pthread_rdwr_rlock_np (&domainlock);
 	    }
