@@ -1,4 +1,4 @@
-/* Copyright (c) 1998, 1999, 2001 Thorsten Kukuk
+/* Copyright (c) 1998, 1999, 2001, 2002 Thorsten Kukuk
    This file is part of ypbind-mt.
    Author: Thorsten Kukuk <kukuk@suse.de>
 
@@ -72,6 +72,7 @@ int ypset = SET_NO;
 int use_broadcast = 0;
 int broken_server = 0;
 int ping_interval = 20;
+int local_only = 0;
 int port = -1;
 static int lock_fd;
 static int pid_is_written = 0;
@@ -101,12 +102,13 @@ unlink_bindingdir (void)
 
 /* Load the config file (/etc/yp.conf)  */
 static int
-load_config (int do_add)
+load_config (int check_syntax)
 {
   FILE *fp;
   char *buf = NULL;
   size_t buflen = 0;
   int have_entries = 0; /* # of entries we found in config file */
+  int bad_entries = 0;
 
   fp = fopen (configfile, "r");
   if (NULL == fp)
@@ -156,7 +158,10 @@ load_config (int do_add)
 	cp[strlen (cp) - 1] = '\0';
 
       if (debug_flag)
-        log_msg (LOG_DEBUG, _("Trying entry: %s"), cp);
+        log_msg (LOG_DEBUG, "%s %s", _("Trying entry:"), cp);
+
+      if (check_syntax)
+	printf ("%s %s\n", _("Trying entry:"), cp);
 
       if (strncmp (cp, "domain", 6) == 0 && isspace ((int)cp[6]))
 	{
@@ -172,9 +177,11 @@ load_config (int do_add)
 	      if (debug_flag)
 		log_msg (LOG_DEBUG, _("parsed domain '%s' server '%s'"),
 			 tmpdomain, tmpserver);
-	      ++have_entries;
-	      if (do_add)
-		add_server (tmpdomain, tmpserver);
+	      if (add_server (tmpdomain, tmpserver, check_syntax))
+		++have_entries;
+	      else
+		++bad_entries;
+
 	      continue;
 	    }
 	  count = sscanf (cp, "domain %s broadcast", tmpdomain);
@@ -183,9 +190,11 @@ load_config (int do_add)
 	      if (debug_flag)
 		log_msg (LOG_DEBUG, _("parsed domain '%s' broadcast"),
 			 tmpdomain);
-	      if (do_add)
-		add_server (tmpdomain, NULL);
-	      ++have_entries;
+	      if (add_server (tmpdomain, NULL, check_syntax))
+		++have_entries;
+	      else
+		++bad_entries;
+
 	      continue;
 	    }
 	}
@@ -199,19 +208,40 @@ load_config (int do_add)
 	    {
 	      if (debug_flag)
 		log_msg (LOG_DEBUG, _("parsed ypserver %s"), tmpserver);
-	      if (do_add)
-		add_server (domain, tmpserver);
-	      ++have_entries;
+	      if (add_server (domain, tmpserver, check_syntax))
+		++have_entries;
+	      else
+		++bad_entries;
 	      continue;
 	    }
 	}
-      log_msg (LOG_ERR, _("Entry \"%s\" is not valid, ignore it!"), cp);
+      if (check_syntax)
+	{
+	  fprintf (stderr, _("Entry \"%s\" is not valid!\n"), cp);
+	  ++bad_entries;
+	}
+      else
+	log_msg (LOG_ERR, _("Entry \"%s\" is not valid, ignore it!"), cp);
     }
   fclose (fp);
 
   if (buf)
     free (buf);
 
+  if (check_syntax)
+    {
+      if (bad_entries)
+	{
+	  fprintf (stderr, _("Bad entries found.\n"));
+	  return 1;
+	}
+      if (!have_entries)
+	{
+	  fprintf (stderr, _("No entry found.\n"));
+	  return 1;
+	}
+    }
+
   if (!have_entries)
     {
       if (debug_flag)
@@ -369,9 +399,9 @@ sig_handler (void *v_param  __attribute__ ((unused)))
 	  clear_server ();
 
 	  if (use_broadcast)
-	    add_server (domain, NULL);
+	    add_server (domain, NULL, 0);
 	  else
-	    load_config (1);
+	    load_config (0);
 
 	  if (ping_interval < 1)
 	    do_binding ();
@@ -387,7 +417,7 @@ static void
 usage (void)
 {
   fputs (_("Usage:\n"), stderr);
-  fputs (_("\typbind [-broadcast | -ypset | -ypsetme] [-p port] [-f configfile]\n\t  [-no-ping] [-broken-server] [-debug]\n"), stderr);
+  fputs (_("\typbind [-broadcast | -ypset | -ypsetme] [-p port] [-f configfile]\n\t  [-no-ping] [-broken-server] [-local_only] [-debug]\n"), stderr);
   fputs (_("\typbind -c [-f configfile]\n"), stderr);
   fputs (_("\typbind --version\n"), stderr);
   exit (1);
@@ -432,6 +462,9 @@ main (int argc, char **argv)
 	ping_interval = 0;
       else if (strcmp ("-broadcast", argv[i]) == 0)
 	use_broadcast = 1;
+      else if (strcmp ("-local_only", argv[i]) == 0 ||
+	       strcmp ("-local-only", argv[i]) == 0)
+	local_only = 1;
       else if (strcmp ("-f", argv[i]) == 0)
 	{
 	  if (i+1 == argc || argv[i+1][0] == '-')
@@ -452,11 +485,28 @@ main (int argc, char **argv)
 	usage ();
     }
 
+  if (yp_get_default_domain (&domain) || domain == NULL ||
+      domain[0] == '\0' || strcmp(domain, "(none)") == 0)
+    {
+      if (configcheck_only)
+	{
+	  fputs (_("ERROR: domainname not set.\n"), stderr);
+	  domain = "(none)";
+	}
+      else
+	{
+	  fputs (_("domainname not set - aborting.\n"), stderr);
+	  exit (1);
+	}
+    }
+
   if (configcheck_only)
     {
-      debug_flag = 1;
-      if (load_config (0) != 0)
-	exit (1);
+      if (load_config (1) != 0)
+	{
+	  fprintf (stderr, _("Config file %s is not ok.\n"), configfile);
+	  exit (1);
+	}
       else
 	{
 	  fprintf (stdout, _("Config file %s is ok.\n"), configfile);
@@ -464,13 +514,6 @@ main (int argc, char **argv)
 	}
     }
 
-  if (yp_get_default_domain (&domain) || domain == NULL ||
-      domain[0] == '\0' || strcmp(domain, "(none)") == 0)
-    {
-      fputs (_("domainname not set - aborting.\n"), stderr);
-      exit (1);
-    }
-
   if (getuid() != 0)
     {
       fputs (_("ypbind must be run as root\n"), stderr);
@@ -504,7 +547,7 @@ main (int argc, char **argv)
 
   if (!use_broadcast)
     {
-      if (load_config (1) != 0)
+      if (load_config (0) != 0)
 	{
 	  fputs (_("No NIS server and no -broadcast option specified.\n"), stderr);
 	  fprintf (stderr,
@@ -515,7 +558,7 @@ main (int argc, char **argv)
 	}
     }
   else
-    add_server (domain, NULL);
+    add_server (domain, NULL, 0);
 
   unlink_bindingdir ();
 
@@ -592,7 +635,7 @@ main (int argc, char **argv)
   pmap_unset (YPBINDPROG, YPBINDOLDVERS);
   pmap_unset (YPBINDPROG, YPBINDVERS);
 
-  if (port >= 0)
+  if (port >= 0 || local_only)
     {
       sock = socket (AF_INET, SOCK_DGRAM, 0);
       if (sock < 0)
@@ -603,8 +646,13 @@ main (int argc, char **argv)
 
       memset ((char *) &socket_address, 0, sizeof (socket_address));
       socket_address.sin_family = AF_INET;
-      socket_address.sin_addr.s_addr = htonl (INADDR_ANY);
-      socket_address.sin_port = htons (port);
+      if (local_only)
+	socket_address.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
+      else
+	{
+	  socket_address.sin_addr.s_addr = htonl (INADDR_ANY);
+	  socket_address.sin_port = htons (port);
+	}
 
       result = bind (sock, (struct sockaddr *) &socket_address,
 		     sizeof (socket_address));
@@ -640,7 +688,7 @@ main (int argc, char **argv)
       exit (1);
     }
 
-  if (port >= 0)
+  if (port >= 0 || local_only)
     {
       sock = socket (AF_INET, SOCK_STREAM, 0);
       if (sock < 0)
@@ -651,8 +699,13 @@ main (int argc, char **argv)
 
       memset (&socket_address, 0, sizeof (socket_address));
       socket_address.sin_family = AF_INET;
-      socket_address.sin_addr.s_addr = htonl (INADDR_ANY);
-      socket_address.sin_port = htons (port);
+      if (local_only)
+	socket_address.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
+      else
+	{
+	  socket_address.sin_addr.s_addr = htonl (INADDR_ANY);
+	  socket_address.sin_port = htons (port);
+	}
 
       result = bind (sock, (struct sockaddr *) &socket_address,
 		     sizeof (socket_address));
